function [ti] = eff2mat(fn, type, verbose, ti_struct)
% Convert EFF file to matlab struct
% [ti] = eff2mat(fn, type, verbose, [ti_struct])
% ti.stain
% ti.files
% fn = filename
% type = EFF type, currently only 'mesh' supported
% verbose:  0 quiet
%           1 show progress "bar" (recommended)
%           2 show progress for every EFF entry
% ti_struct = prefilled ti for return ti (optional)
%
% e.g.
% tf = eff2mat('/Users/erwin/Desktop/TF_201204dump.eff','mesh',1);

    preallocate = 1;

    fid = fopen(fn);
    if fid < 0,
        error('Cannot open ''%s''', fn);
    end
    
    effline = fgetl(fid);
    
    if ~strcmp(effline, 'EFF 1.0')
        fclose(fid);
        error('Not an EFF file, identifier missing - found %s instead', effline);
    end

    % Copy contents of ti_struct to returned ti
    if exist('ti_struct', 'var'),
        ti = ti_struct;
    end
    
    if preallocate == 1,
        % soo much faster after ~1000 entries - not even funny
        
        
        if verbose > 0, fprintf('Preallocating variables ...'); end
        
        ct = 0;
        line = fgetl(fid);
        entries=strread(line,'%s','delimiter','\t');
        
        d_ct = entries{6};

        while ischar(line),
            ct = ct+1;
            line = fgetl(fid);
        end
        
        ti.stain = zeros(str2num(d_ct),ct);
        ti.files = cell(ct,1);
 
        if verbose > 0, fprintf(' done, found %d entries\n', ct); end
        
        fclose(fid);
        fid = fopen(fn);
        line = fgetl(fid);
        
    else
        ti.stain = [];
        ti.files = {};
    end
    
    ct = 1;
    line = fgetl(fid);
    while ischar(line),
        
        if verbose > 1, fprintf('%d: ', ct); end
        
        entries=strread(line,'%s','delimiter','\t');
        
        file = entries{1};
        d_type = entries{3};
        d_ct = entries{6};
        data = entries{8};
        
        if ~strcmp(d_type, type),
            if verbose == 0, fprintf('%d: ', ct); end
            fprintf('WARNING: Wrong type (''%s'') - skipping\n', d_type);
            continue;
        end
        
        % separate into tokens
        st=strread(data,'%n','delimiter',',');
        if str2num(d_ct) ~= length(st)
            if verbose == 0, fprintf('%d: ', ct); end
            fprintf('WARNING: Wrong data length (%d, expected %d) - skipping\n', length(st), str2num(d_ct));
            continue;
        end
            
        ti.files{ct} = file;
        if preallocate == 1,
            ti.stain(:,ct) = st;
        else
            ti.stain = [ti.stain, st];
        end
        ct = ct + 1;
        
        if verbose > 0,
            if verbose > 1, 
                fprintf('%s OK\n', file);
            else
                if mod(ct,500) == 0, 
                    fprintf('%d\n',ct);
                elseif mod(ct,100) == 0, 
                    fprintf('%d',ct);
                elseif mod(ct,20) == 0,
                    fprintf('.');
                end
            end
        end    
            
               
        line = fgetl(fid);
    end

    if verbose == 1, fprintf('\n'); end
    
    if preallocate == 1 && ct - 1 ~= length(ti.files),
        fprintf('Length different from pre-allocation, some lines may be skipped?\n');
        fprintf('Adjusting length...\n');
        ti.stain = ti.stain(:,1:ct-1);
        ti.files = ti.files(1:ct-1); 
    end

    fclose(fid);
end